package org.kapelle.multiblock;

import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;

import java.util.Collections;
import java.util.Set;

interface CheckFunction {
    boolean apply(Location location);
}

public class BlockComponent {
    private final BlockComponentMode mode;
    private final Set<Material> blocks;
    private CheckFunction customCheckFunction;

    public BlockComponent(BlockComponentMode mode){
        if(mode == BlockComponentMode.SET){
            throw new RuntimeException("Component is impossible to satisfy because no valid blocks are given");
        }

        this.mode = mode;
        this.blocks = Collections.emptySet();
    }

    public BlockComponent(BlockComponentMode mode, Set<Material> validBlocks){
        this.mode = mode;
        this.blocks = validBlocks;
    }

    public BlockComponent(CheckFunction customCheckFunction){
        this.mode = BlockComponentMode.CUSTOM;
        this.blocks = Collections.emptySet();
        this.customCheckFunction = customCheckFunction;
    }

    public boolean isValidBlock(Location location){
        if(this.mode == BlockComponentMode.CUSTOM){
            return this.customCheckFunction.apply(location);
        }else{
            return isValidType(location.getBlock().getType());
        }
    }

    private boolean isValidType(Material type){
        switch (this.mode){
            case AIR:
                return type == Material.AIR;
            case ANY:
                return true;
            case SET:
                return blocks.contains(type);
            default:
                return false;

        }
    }

}
